#include <gint/display.h>
#include <gint/keyboard.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "list.h"
#include "MyKeyboard.h"

list* langues;

extern bopti_image_t BoxText;
extern bopti_image_t Button;
extern bopti_image_t ButtonSwitch;
extern bopti_image_t Logo;

int LangueFrom = 0;
int LangueTo = 1;

void DrawRect(int x, int y, int w, int h, int color){
    dline(x, y, x+w, y, color);
    dline(x, y, x, y+h, color);
    dline(x+w, y, x+w, y+h, color);
    dline(x, y+h, x+w, y+h, color);
}

int IsKeyDownWait(int key){
    if (keydown(key)){
        while (keydown(key)){
            clearevents();
        }
        return 1;
    }
    return 0;
}

#define MAX_BUTTON 6
int ButtonSelected = 0;

void DrawUI(){
    int ButtonPos[MAX_BUTTON][4] = {
    {4, 65, 110, 85},
    {186+4, 65, 186+110, 85},
    {4, 87,BoxText.width-25, 117},
    {4, 117,BoxText.width-25, 157},
    {28, 170, 28+ButtonSwitch.width, 170+ButtonSwitch.height},
    {134, 169, 134+Button.width, 169+Button.height}};

    dimage(98, 11, &Logo);
    dimage(0, 62, &BoxText);
    dimage(186, 62, &BoxText);
    dimage(134, 169, &Button);
    dtext(134+10, 169+10, C_BLACK, "Tranduire");
    dimage(28, 170, &ButtonSwitch);

    dtext(4+5,65+5, C_BLACK, list_get(langues, LangueFrom));
    dtext(186+4+5,65+5, C_BLACK, list_get(langues, LangueTo));

    if (IsKeyDownWait(KEY_RIGHT) || IsKeyDownWait(KEY_DOWN)){
        ButtonSelected = (ButtonSelected + 1) % MAX_BUTTON;
    }
    if (IsKeyDownWait(KEY_LEFT) || IsKeyDownWait(KEY_UP)){
        ButtonSelected = (ButtonSelected + MAX_BUTTON - 1) % MAX_BUTTON;
    }
    DrawRect(ButtonPos[ButtonSelected][0], ButtonPos[ButtonSelected][1], ButtonPos[ButtonSelected][2]-ButtonPos[ButtonSelected][0], ButtonPos[ButtonSelected][3]-ButtonPos[ButtonSelected][1], C_BLACK);
}

/*dtext(0, 0, C_WHITE, (const char*)text);
dupdate();
keydown(KEY_EXE)
dclear(C_BLACK);
clearevents();*/

char* ReadWord(FILE* fp) {
    short sizeWord;
    fread(&sizeWord, sizeof sizeWord, 1, fp);
    char* word = (char*)malloc(sizeWord + 1);
    for (int i = 0; i < sizeWord; i++) {
        fread(&word[i], sizeof word[i], 1, fp);
    }
    word[sizeWord] = '\0';
    //to lower
    for (int i = 0; i < sizeWord; i++) {
        if (word[i] >= 'A' && word[i] <= 'Z') {
            word[i] = word[i] - 'A' + 'a';
        }
    }
    return word;
}

void GetAllLangue(list** langues) {
    *langues = list_new();
    FILE* fp;
    fp = fopen("dico.bin", "r");
    short sizeLangue;
    fread(&sizeLangue, sizeof sizeLangue, 1, fp);
    for (int i = 0; i < sizeLangue; i++) {
        char* langue = ReadWord(fp);
        list_add(*langues, langue);
    }
    fclose(fp);
}

int GetIndexLangue(char* langue) {
    for (int i = 0; i < langues->size; i++) {
        if (strcmp(langue, (char*)list_get(langues, i)) == 0) {
            return i;
        }
    }
    return -1;
}

char** ReadPackWord(FILE* fp) {
    char** PackWord = (char**)malloc(sizeof(char*) * langues->size);
    for (int j = 0; j < langues->size; j++) {
        PackWord[j] = ReadWord(fp);
    }
    return PackWord;
}

void FreePackWord(char** PackWord) {
    for (int i = 0; i < langues->size; i++) {
        free(PackWord[i]);
    }
    free(PackWord);
}

char* TranslateWord(char* langueFrom, char* langueTo, char* word) {
    //to lower
    for (int i = 0; i < strlen(word); i++) {
        if (word[i] >= 'A' && word[i] <= 'Z') {
            word[i] = word[i] - 'A' + 'a';
        }
    }
    int indexFrom = GetIndexLangue(langueFrom);
    int indexTo = GetIndexLangue(langueTo);
    if (indexFrom == -1 || indexTo == -1) {
        return NULL;
    }
    FILE* fp;
    fp = fopen("dico.bin", "r");
    short sizeLangue;
    fread(&sizeLangue, sizeof sizeLangue, 1, fp);
    for (int i = 0; i < sizeLangue; i++) {
        char* langue = ReadWord(fp);
        free(langue);
    }
    short NbWord;
    fread(&NbWord, sizeof NbWord, 1, fp);
    char** PackWord = (char**)malloc(sizeof(char*) * langues->size);
    for (int i = 0; i < NbWord; i++) {
        char** PackWord = ReadPackWord(fp);
        if (strcmp(PackWord[indexFrom], word) == 0) {
            char* temp = (char*)malloc(sizeof(char) * (strlen(PackWord[indexTo]) + 1));
            strcpy(temp, PackWord[indexTo]);
            FreePackWord(PackWord);
            fclose(fp);
            return temp;
        }
        FreePackWord(PackWord);
    }
    fclose(fp);
    return NULL;
}

int IsInList(list* listWord, char* word) {
    for(list_node* cur= NULL;ForEach(listWord, &cur);)
    {
        if (strcmp(word, (char*)cur->data) == 0) {
            return 1;
        }
    }
    return 0;
}

void FindProximite(char* langueFrom, char* word,list* listWord) {
    //to lower
    for (int i = 0; i < strlen(word); i++) {
        if (word[i] >= 'A' && word[i] <= 'Z') {
            word[i] = word[i] - 'A' + 'a';
        }
    }
    int indexFrom = GetIndexLangue(langueFrom);
    if (indexFrom == -1) {
        return NULL;
    }
    FILE* fp;
    fp = fopen("dico.bin", "r");
    short sizeLangue;
    fread(&sizeLangue, sizeof sizeLangue, 1, fp);
    for (int i = 0; i < sizeLangue; i++) {
        char* langue = ReadWord(fp);
        free(langue);
    }
    short NbWord;
    fread(&NbWord, sizeof NbWord, 1, fp);
    //free list
    while (listWord->size > 0) {
        free(list_pop(listWord));
    }
    list_clear(listWord);
    for (int i = 0; i < NbWord; i++) {
        if (listWord->size >= 4) {
            fclose(fp);
            return;
        }
        char** PackWord = ReadPackWord(fp);
        //if contain word
        char firstLetterPackWord = PackWord[indexFrom][0];
        if (firstLetterPackWord != word[0]) {
            FreePackWord(PackWord);
            continue;
        }
        if (strstr(PackWord[indexFrom], word) != NULL && !IsInList(listWord, PackWord[indexFrom])) {
            char* temp = (char*)malloc(sizeof(char) * (strlen(PackWord[indexFrom]) + 1));
            strcpy(temp, PackWord[indexFrom]);
            temp[strlen(PackWord[indexFrom])] = '\0';
            list_add(listWord, temp);
        }
        FreePackWord(PackWord);
    }
    fclose(fp);
    return;
}

int TextSelection(list* textes){
    int index = 0;
    IsKeyDownWait(KEY_MENU);
    IsKeyDownWait(KEY_EXE);
    while (!keydown(KEY_MENU))
    {
        dclear(C_WHITE);
        clearevents();
        for (int i = 0; i < textes->size; i++) {
            dtext(20, 10 * i, C_BLACK, (const char*)list_get(textes, i));
        }
        dtext(0, 10 * index, C_BLACK, ">");
        if (IsKeyDownWait(KEY_DOWN)) {
            index++;
            if (index >= textes->size) {
                index = 0;
            }
        }
        if (IsKeyDownWait(KEY_UP)) {
            index--;
            if (index < 0) {
                index = textes->size - 1;
            }
        }
        if (IsKeyDownWait(KEY_EXE)) {
            return index;
        }
        dupdate();
    }
    return 0;
}

char* InputText;
char* OutputText;
list* proximites;

void ValidButton(){
    if (!keydown(KEY_EXE)) {
        return;
    }
    switch (ButtonSelected)
    {
        case 0:
            //select langue from
            int index = TextSelection(langues);
            LangueFrom = index;
            break;
        case 1:
            //select langue to
            int index2 = TextSelection(langues);
            LangueTo = index2;
            break;
        case 3:
            //trasfer proxi to input
            if (proximites->size > 0) {
                int index3 = TextSelection(proximites);
                int lentemp = strlen((char*)list_get(proximites, index3));
                char* temp3 = (char*)malloc(sizeof(char) * (lentemp + 1));
                strcpy(temp3, (char*)list_get(proximites, index3));
                temp3[lentemp] = '\0';
                free(InputText);
                InputText = temp3;
                OutputText = realloc(OutputText, sizeof(char) * 1);
                OutputText[0] = '\0';
            }
            break;
        case 4:
            //switch langue
            int temp2 = LangueFrom;
            LangueFrom = LangueTo;
            LangueTo = temp2;
            //input to output
            char* temp4 = InputText;
            InputText = OutputText;
            OutputText = temp4;
            break;
        case 5:
            //translate
            char* temp = (char*)gint_world_switch(GINT_CALL(TranslateWord,list_get(langues, LangueFrom), list_get(langues, LangueTo), InputText));
            if (temp == NULL)
            {
                    OutputText = realloc(OutputText, sizeof(char) * 1);
                    OutputText[0] = '\0';
            }
            else
            {
                free(OutputText);
                OutputText = temp;
            }
            break;
    }
    IsKeyDownWait(KEY_EXE);
}

int main(void)
{
    gint_world_switch(GINT_CALL(GetAllLangue,(void*)&langues));
    InputText = (char*)malloc(sizeof(char)*1);
    OutputText = (char*)malloc(sizeof(char)*1);
    InputText[0] = '\0';
    OutputText[0] = '\0';
    proximites = list_new();
    while (!keydown(KEY_MENU))
    {
        dclear(C_WHITE);
        DrawUI();
        int size = strlen(InputText);
        if (ButtonSelected==2)
            InputText = scanf_keyboard(InputText);
        int size2 = strlen(InputText);
        if (size != size2) {
            //lock size to 20 char max
            if (size2 > 20) {
                InputText[20] = '\0';
            }
            //text change
            gint_world_switch(GINT_CALL(FindProximite,list_get(langues, LangueFrom), InputText, (void*)proximites));
        }

        dtext(4, 89, C_BLACK, (const char*)InputText);
        dtext(4+186, 89, C_BLACK, (const char*)OutputText);

        //draw list
        for (int i = 0; i < proximites->size; i++) {
            dtext(4, 120 + 10 * i, C_BLACK, (const char*)list_get(proximites, i));
        }
        clearevents();
        ValidButton();
        dupdate();
    }
    
    list_free(langues);
    free(InputText);
    return 1;
}

